#!/usr/bin/perl -w

use strict;
use warnings;
use Inca::DepotClient;
use Inca::DummyServer;
use Test::More 'no_plan';

my $CONFIG = '<inca/>';
my $GUIDS = "guid1\nguid2\nguid3";
my $PORT = 7890;
my @QUERY_HQL_REPLY = (
'<instance>
  <collected>2006-10-09 09:10:03.0</collected>
  <committed>2006-10-09 09:10:03.927</committed>
  <cpu>0.16</cpu>
  <memory>0.0</memory>
  <wall>0.365195</wall>
</instance>
',
'<instance>
  <collected>2006-10-09 09:10:04.0</collected>
  <committed>2006-10-09 09:10:05.007</committed>
  <cpu>0.69</cpu>
  <memory>9.1171875</memory>
  <wall>1.752196</wall>
</instance>
',
'<instance>
  <collected>2006-10-09 09:15:02.0</collected>
  <committed>2006-10-09 09:15:04.506</committed>
  <cpu>0.15</cpu>
  <memory>0.0</memory>
  <wall>0.416581</wall>
</instance>
');
my $QUERY_INSTANCE_REPLY =
'<reportDetails xmlns="http://inca.sdsc.edu/queryResult/reportDetails_2.0">
  <suiteId xmlns="">1</suiteId>
  <seriesConfigId xmlns="">1</seriesConfigId>
  <seriesId xmlns="">1</seriesId>
  <reportId xmlns="">1</reportId>
  <instanceId xmlns="">39</instanceId>
  <seriesConfig xmlns="">
    <series>
      <name>cluster.compiler.gcc.version</name>
      <version>1.5</version>
      <uri>file:///Users/jhayes/devel/reporters/bin/cluster.compiler.gcc.version</uri>
      <args>
        <arg>
          <name>version</name>
          <value>no</value>
        </arg>
        <arg>
          <name>help</name>
          <value>no</value>
        </arg>
        <arg>
          <name>log</name>
          <value>0</value>
        </arg>
        <arg>
          <name>verbose</name>
          <value>1</value>
        </arg>
      </args>
      <context>cluster.compiler.gcc.version -help="no" -log="0" -verbose="1" -version="no"</context>
      <nice>false</nice>
    </series>
    <nickname>cluster.compiler.gcc.version</nickname>
    <resourceHostname>localhost</resourceHostname>
    <schedule>
      <cron>
        <min>*/5</min>
        <hour>*</hour>
        <mday>*</mday>
        <wday>*</wday>
        <month>*</month>
        <numOccurs>-1</numOccurs>
        <suspended>false</suspended>
      </cron>
    </schedule>
    <action>add</action>
  </seriesConfig>
  <report xmlns="">
    <gmt>2006-10-09T10:45:03.000-07:00</gmt>
    <hostname>client64-120.sdsc.edu</hostname>
    <name>cluster.compiler.gcc.version</name>
    <version>1.5</version>
    <workingDir>/private/tmp/jjhinstall</workingDir>
    <reporterPath>/private/tmp/jjhinstall/var/reporter-packages/bin/cluster.compiler.gcc.version-1.5</reporterPath>
    <args>
      <arg>
        <name>version</name>
        <value>no</value>
      </arg>
      <arg>
        <name>help</name>
        <value>no</value>
      </arg>
      <arg>
        <name>log</name>
        <value>0</value>
      </arg>
      <arg>
        <name>verbose</name>
        <value>1</value>
      </arg>
    </args>
    <body>
      <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
        <ID>gcc</ID>
        <version>3.3</version>
      </package>
    </body>
    <exitStatus>
      <completed>true</completed>
      <errorMessage/>
    </exitStatus>
  </report>
  <sysusage xmlns="">
    <wallClockTime>0.321577</wallClockTime>
    <memory>0.0</memory>
    <cpuTime>0.11</cpuTime>
  </sysusage>
  <stderr xmlns=""/>
</reportDetails>';
my @QUERY_LATEST_REPLY = (
'<reportSummary xmlns="http://inca.sdsc.edu/queryResult/reportSummary_2.0">
  <hostname xmlns="">localhost</hostname>
  <uri xmlns="">file:///Users/jhayes/devel/reporters/bin/cluster.compiler.gcc.version</uri>
  <nickname xmlns="">cluster.compiler.gcc.version</nickname>
  <seriesConfigId xmlns="">1</seriesConfigId>
  <instanceId xmlns="">5</instanceId>
  <body xmlns="">
    <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
      <ID>gcc</ID>
      <version>3.3</version>
    </package>
  </body>
</reportSummary>',
'<reportSummary xmlns="http://inca.sdsc.edu/queryResult/reportSummary_2.0">
  <hostname xmlns="">localhost</hostname>
  <uri xmlns="">file:///Users/jhayes/devel/reporters/bin/cluster.admin.ant.version</uri>
  <nickname xmlns="">cluster.admin.ant.version</nickname>
  <seriesConfigId xmlns="">2</seriesConfigId>
  <instanceId xmlns="">6</instanceId>
  <body xmlns="">
    <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
      <ID>ant</ID>
      <version>1.6.5</version>
    </package>
  </body>
</reportSummary>'
);
my @QUERY_PERIOD_REPLY = (
 '<object>
    <resource>local</resource>
    <nickname>reporter.a</nickname>
    <instanceId>1234</instanceId>
    <reportId>2345</reportId>
    <configId>3456</configId>
    <collected>2006-10-09 09:15:02.0</collected>
    <exit_status>Success</exit_status>
    <exit_message> </exit_message>
    <comparisonResult>Success</comparisonResult>
  </object>',
 '<object>
    <resource>local</resource>
    <nickname>reporter.b</nickname>
    <instanceId>4321</instanceId>
    <reportId>5432</reportId>
    <configId>6543</configId>
    <collected>2006-10-09 19:15:02.0</collected>
    <exit_status>Success</exit_status>
    <exit_message> </exit_message>
    <comparisonResult>Success</comparisonResult>
  </object>'
);
my @QUERY_STATUS_REPLY = (
'<series>
   <guid>guid1</guid>
   <nickname>reporter.a</nickname>
   <resource>local</resource>
   <period>
     <begin>1234567</begin>
     <end>2345678</end>
     <success>22</success>
     <failure>
       <message>This is a failure</message>
       <count>33</count>
     </failure>
   </period>
 </series>',
'<series>
   <guid>guid1</guid>
   <nickname>reporter.b</nickname>
   <resource>local</resource>
   <period>
     <begin>1234567</begin>
     <end>2345678</end>
     <success>44</success>
     <failure>
       <message>This is another failure</message>
       <count>55</count>
     </failure>
     <failure>
       <message>And yet a third</message>
       <count>66</count>
     </failure>
   </period>
   <period>
     <begin>3456789</begin>
     <end>4567890</end>
     <success>11</success>
     <failure>
       <message>This is another failure</message>
       <count>12</count>
     </failure>
   </period>
 </series>'
);
my $UPDATE_SUITE_DATA =
'<inca:suite xmlns:inca="http://inca.sdsc.edu/dataModel/inca_2.0> 
  <seriesConfigs>
    <seriesConfig>
      <series>
        <name>reporter.a</name>
        <version>1.2</version>
        <args>
          <arg>
            <name>version</name>
            <value>no</value>
          </arg>
          <arg>
            <name>verbose</name>
            <value>1</value>
          </arg>
          <arg>
            <name>help</name>
            <value>no</value>
          </arg>
          <arg>
            <name>log</name>
            <value>0</value>
          </arg>
        </args>
        <context>reporter.a -version=no -verbose=1 -help=no -log=0</context>
      </series>
      <nickname>reporter.a</nickname>
      <resourceSetName>local</resourceSetName>
      <schedule>
        <cron>
          <min>6</min>
          <hour>1/1</hour>
          <mday>*</mday>
          <wday>*</wday>
          <month>*</month>
        </cron>
      </schedule>
      <action>add</action>
    </seriesConfig>
    <seriesConfig>
      <series>
        <name>reporter.b</name>
        <args>
          <arg>
            <name>version</name>
            <value>no</value>
          </arg>
          <arg>
            <name>verbose</name>
            <value>1</value>
          </arg>
          <arg>
            <name>help</name>
            <value>no</value>
          </arg>
          <arg>
            <name>log</name>
            <value>0</value>
          </arg>
        </args>
        <context>reporter.b -version=no -verbose=1 -help=no -log=0</context>
      </series>
      <nickname>reporter.b</nickname>
      <resourceSetName>local</resourceSetName>
      <schedule>
        <cron>
          <min>20</min>
          <hour>1/1</hour>
          <mday>*</mday>
          <wday>*</wday>
          <month>*</month>
        </cron>
      </schedule>
      <action>add</action>
    </seriesConfig>
  </seriesConfigs>
  <name>aSuite</name>
  <guid/>
  <description/>
  <version>1</version>
</inca:suite>
';
my $TEST_DATA = 'hello, sailor!';
my $TEST_VERSION = '1.0';

my $serverPid = fork();
if($serverPid == 0) {
  my $server = new Inca::DummyServer($PORT,
    'QUERYGUIDS'         => "'OK $GUIDS'",
    'QUERYHQL (.+)'      =>
      "'QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_HQL_REPLY) .
      "\r\nQUERYEND'",
    'QUERYINSTANCE (\d+) (\d+)' =>
      "'QUERYRESULT $QUERY_INSTANCE_REPLY\r\nQUERYEND'",
    'QUERYSQL (.+)'      => "'ERROR TODO'",
    'QUERYLATEST (.+)'   =>
      "'QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_LATEST_REPLY) .
      "\r\nQUERYEND'",
    'QUERYPERIOD (.+)'   =>
      "'QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_PERIOD_REPLY) .
      "\r\nQUERYEND'",
    'QUERYSTATUS (.+)'   =>
      "'QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_STATUS_REPLY) .
      "\r\nQUERYEND'",
    'SUITE (.+)'         => "'OK 2'",
    'PING (.+)'          => '"OK $1"',
    'START (.+)'         => '"OK $1"'
  );
  $server->run();
  exit(0);
}

my $client;
# Give the server a chance to start
sleep(3);
for(my $i = 0; $i < 10; $i++) {
  $client =
    new Inca::DepotClient(host => 'localhost', port => $PORT, auth => 0);
  last if defined($client);
  sleep(1);
}

my ($err, $reply);
$reply = $client->ping($TEST_DATA);
$err = $client->getError();
ok(defined($reply), 'PING responded ...');
ok(!defined($err), ' ... no error ' . (defined($err) ? "'$err'" : ''));
ok($reply eq $TEST_DATA, '... data content "' . $reply . '"');

$reply = $client->start($TEST_VERSION);
$err = $client->getError();
ok(defined($reply), 'START responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok($reply eq $TEST_VERSION, '... data content "' . $reply . '"');

$reply = $client->queryGuids();
$err = $client->getError();
ok(defined($reply), 'QUERYGUIDS responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok($reply eq $GUIDS, '... data content "' . $reply . '"');

$reply = $client->updateSuite($UPDATE_SUITE_DATA);
$err = $client->getError();
ok(defined($reply), 'SUITE responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));

$reply = $client->queryHql('select ii from InstanceInfo as ii');
$err = $client->getError();
ok(defined($reply), 'QUERYHQL responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok(ref($reply) && ref($reply) eq 'ARRAY', '... reply is an array reference');
ok($#{$reply} == $#QUERY_HQL_REPLY, '... correct reply length ' . $#{$reply});
$reply = join("\n", @$reply);
ok($reply eq join("\n", @QUERY_HQL_REPLY), "... data content '$reply'");

$reply = $client->queryInstance("cluster.compiler.gcc.version", "local", "2006-10-09T10:45:03.000-07:00");
$err = $client->getError();
ok(defined($reply), 'QUERYINSTANCE responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok(ref($reply) && ref($reply) eq 'ARRAY', '... reply is an array reference');
ok($#{$reply} == 0, '... correct reply length ' . $#{$reply});
$reply = join('', @$reply);
ok($reply eq $QUERY_INSTANCE_REPLY, "... data content '$reply'");

$reply = $client->queryLatest("suite.guid = 'guid1'");
$err = $client->getError();
ok(defined($reply), 'QUERYLATEST responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok(ref($reply) && ref($reply) eq 'ARRAY', '... reply is an array reference');
ok($#{$reply} == $#QUERY_LATEST_REPLY, '... correct reply length ' . $#{$reply});
$reply = join("\n", @$reply);
ok($reply eq join("\n", @QUERY_LATEST_REPLY), "... data content '$reply'");

$reply = $client->queryPeriod(1234567, 4567890, "suite.guid = 'guid1'");
$err = $client->getError();
ok(defined($reply), 'QUERYPERIOD responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok(ref($reply) && ref($reply) eq 'ARRAY', '... reply is an array reference');
ok($#{$reply} == $#QUERY_PERIOD_REPLY, '... correct reply length ' . $#{$reply});
$reply = join("\n", @$reply);
ok($reply eq join("\n", @QUERY_PERIOD_REPLY), "... data content '$reply'");

$reply = $client->queryStatus('WEEK', 1234567, 4567890, "suite.guid = 'guid1'");
$err = $client->getError();
ok(defined($reply), 'QUERYSTATUS responded ...');
ok(!defined($err), '... no error ' . (defined($err) ? "'$err'" : ''));
ok(ref($reply) && ref($reply) eq 'ARRAY', '... reply is an array reference');
ok($#{$reply} == $#QUERY_STATUS_REPLY, '... correct reply length ' . $#{$reply});
$reply = join("\n", @$reply);
ok($reply eq join("\n", @QUERY_STATUS_REPLY), "... data content '$reply'");

kill 9, $serverPid;
sleep(3);
